;-------------------------------------------------------------------------------------------
;
; XpressNet/PC interface (mint Lenz LI100F)
;
; v0.9
;
;
; www.tfd.hu
; tfd@tfd.hu
;
; PIC18F1320 processorral
;-------------------------------------------------------------------------------------------



	processor	18f1320


	include		"p18f1320.inc"


  __CONFIG  _CONFIG1H, _IESO_OFF_1H & _FSCM_OFF_1H & _HSPLL_OSC_1H
;	Oscillator switchover disable, Fail-safe off, HS oscillator with PLL

  __CONFIG  _CONFIG2L, _PWRT_ON_2L & _BOR_ON_2L & _BORV_45_2L
;	Power-up timer enabled, Brown-out reset enabled, Vbor=4.5V

  __CONFIG  _CONFIG2H, _WDT_OFF_2H & _WDTPS_32K_2H
;	Watchdog timer disabled, WDTPS=1:32768

  __CONFIG  _CONFIG3H, _MCLRE_OFF_3H
;	RA5 as an input, MCLRE disabled

  __CONFIG  _CONFIG4L, _DEBUG_OFF_4L & _LVP_ON_4L & _STVR_OFF_4L
;	debugger off, Low voltage ICSP enabled, Stack under/ower-flow disabled

  __CONFIG  _CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L
;	all program memory code protection is off

  __CONFIG  _CONFIG5H, _CPB_OFF_5H & _CPD_OFF_5H
;	boot block and data EEPROM is not codeprotected

  __CONFIG  _CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L
  __CONFIG  _CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H & _WRTD_OFF_6H
  __CONFIG  _CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L
  __CONFIG  _CONFIG7H, _EBTRB_OFF_7H
;	all memory bloks are readable/writeble


;------ konstansok


OSCFQ		EQU	.40000000	;oszcilltor frekvencia [Hz]
RS232_baud	EQU	.18

;------ portok

#define	UART2_CTS	PORTB, .2
#define	UART2_RX	PORTB, .3
#define	UART2_TX	PORTB, .6

#define LED		PORTB, .7

#define	XN_DE		PORTB, .0

;------ RAM

CBLOCK	0x00

	Cntr1
	Cntr2

	LED_Cntr

	UART2_TXReg
	UART2_TXState
	UART2_RXReg
	UART2_RXState
;	UART2_Baud
;	UART2_BaudCntr


	FROMPC_ToutCntr
	FROMPC_XorByte
	FROMPC_BytesLeft
	FROMPC_BufferPtr
	FROMPC_ToXN

	TOPC_WPtr
	TOPC_RPtr
	TOPC_FSRTemp
	TOPC_DataTemp

	XN_State
	XN_BytesLeft
	XN_BufferPtr
	XN_rcallByte
	XN_Addr
	XN_XorByte

	IRQ_FSRTemp
	IRQ_STATUSTemp
	IRQ_WTemp

ENDC

CBLOCK	0x40

	FROMPC_Buffer:.32
	XN_Buffer:.20
	TOPC_Buffer:.32

ENDC


;------ program



		ORG	0x0000

		clrf	INTCON
		bra	start


;
; TMR0L megszakts
; ----------------
;

;	----	high priority interrupt 
		ORG	0x0008
		bra	high_start

;	----	low priority interrupt
		ORG	0x0018

; nem hasznlt, ugyanaz mint a hihg priority-nl
;		bra	high_start

high_start	

		movwf	IRQ_WTemp			; W mentse
		movff	STATUS, IRQ_STATUSTemp

		movlw	RS232_baud			; 9600 baud
		movwf	TMR0L

		bcf	INTCON, TMR0IF			; megszaktst vettk


;
; Soft UART TX
; ------------
;
; Az utna kvetkez vtel miatt fontos,
; hogy mindg ugyanannyi id alatt fusson le.
;
; Ha UART2_TXState=40, akkor adsra ksz.
; UART2_TXReg be adatot berni, majd UART2_TXState trlsre
; elindul az ads.
;

		movlw	high tx_table
		movwf	PCLATH
		bcf	STATUS, C
		rlcf	UART2_TXState, w
		addlw	low tx_table
		btfsc	STATUS, C
		incf	PCLATH, f
		movwf	PCL

tx_table	

		bra	u2txstb			;0 - start bit ads
		bra	u2txnop			;1
		bra	u2txnop			;2
		bra	u2txnop			;3

		bra	u2txdb			;4 - 0. bit ads
		bra	u2txnop			;5
		bra	u2txnop			;6
		bra	u2txnop			;7

		bra	u2txdb			;8 - 1. bit ads
		bra	u2txnop			;9
		bra	u2txnop			;10
		bra	u2txnop			;11

		bra	u2txdb			;12 - 2. bit ads
		bra	u2txnop			;13
		bra	u2txnop			;14
		bra	u2txnop			;15

		bra	u2txdb			;16 - 3. bit ads
		bra	u2txnop			;17
		bra	u2txnop			;18
		bra	u2txnop			;19

		bra	u2txdb			;20 - 4. bit ads
		bra	u2txnop			;21
		bra	u2txnop			;22
		bra	u2txnop			;23

		bra	u2txdb			;24 - 5. bit ads
		bra	u2txnop			;25
		bra	u2txnop			;26
		bra	u2txnop			;27

		bra	u2txdb			;28 - 6. bit ads
		bra	u2txnop			;29
		bra	u2txnop			;30
		bra	u2txnop			;31

		bra	u2txdb			;32 - 7. bit ads
		bra	u2txnop			;33
		bra	u2txnop			;34
		bra	u2txnop			;35

		bra	u2txspb			;36 - stop bit ads
		bra	u2txnop			;37
		bra	u2txnop			;38
		bra	u2txnop			;39

		bra	u2txrdy			;40 - ads befejezve

u2txstb
		nop
		nop
		nop
		nop
		bcf	UART2_TX
		bra	u2txend

u2txdb
		rrcf	UART2_TXReg, f		;bit kilptets
		btfsc	STATUS,C
		bra	u2tx1
		nop
		bcf	UART2_TX		;kilptetett bit 0
		bra	u2txend
u2tx1		bsf	UART2_TX		;kilptetett bit 1
		bra	u2txend

u2txspb
		nop
		nop
		nop
		nop
		bsf	UART2_TX
		bra	u2txend

u2txnop
		nop
		nop
		nop
		nop
		nop
		bra	u2txend

u2txrdy
		nop
		nop
		nop
		nop
		nop
		nop
		decf	UART2_TXState, f		;maradunk ebben az llapotban

u2txend		incf	UART2_TXState, f		;kvetkez llapot



;
; Soft UART RX
; ------------
;
; Futs ideje nem fontos, nincs utna semmi rdekes...
;
; UART2_RXState trlse engedlyezi a vtelt.
; Ha UART2_TXState=40, akkor a vett adat az UART2_RXReg-ben.
;

		movlw	high rx_table
		movwf	PCLATH
		bcf	STATUS, C
		rlcf	UART2_RXState, w
		addlw	low rx_table
		btfsc	STATUS, C
		incf	PCLATH, f
		movwf	PCL

rx_table	bra	u2rxstb			;0 - start bitre vrs
		bra	u2rxnext		;1
		bra	u2rxnext		;2
		bra	u2rxnext		;3

		bra	u2rxnext		;4
		bra	u2rxnext		;5
		bra	u2rxdb			;6 - 0. bit vtel
		bra	u2rxnext		;7

		bra	u2rxnext		;8
		bra	u2rxnext		;9
		bra	u2rxdb			;10 - 1. bit vtel
		bra	u2rxnext		;11

		bra	u2rxnext		;12
		bra	u2rxnext		;13
		bra	u2rxdb			;14 - 2. bit vtel
		bra	u2rxnext		;15

		bra	u2rxnext		;16
		bra	u2rxnext		;17
		bra	u2rxdb			;18 - 3. bit vtel
		bra	u2rxnext		;19

		bra	u2rxnext		;20
		bra	u2rxnext		;21
		bra	u2rxdb			;22 - 4. bit vtel
		bra	u2rxnext		;23

		bra	u2rxnext		;24
		bra	u2rxnext		;25
		bra	u2rxdb			;26 - 5. bit vtel
		bra	u2rxnext		;27

		bra	u2rxnext		;28
		bra	u2rxnext		;29
		bra	u2rxdb			;30 - 6. bit vtel
		bra	u2rxnext		;31

		bra	u2rxnext		;32
		bra	u2rxnext		;33
		bra	u2rxdb			;34 - 7. bit vtel
		bra	u2rxnext		;35

		bra	u2rxnext		;36
		bra	u2rxnext		;37
		bra	u2rxnext		;38 - stop bitet lehetne vizsglni...
		bra	u2rxnext		;39

		bra	u2rxend			;40 - vtel befejezve


u2rxstb		btfss	UART2_RX		;ha a bemenet alacsony,
		bra	u2rxnext		;akkor kvetkez llapot, mert vtel elindult

		movf	FROMPC_ToutCntr,w	;vtel timeout szmll
		btfss	STATUS,Z		;ha mg nem 0,
		decf	FROMPC_ToutCntr,f	;akkor cskken

		bra	u2rxend			;llapot nem vltozik


u2rxdb		bcf	STATUS,C
		btfsc	UART2_RX		;adatbit
		bsf	STATUS,C
		rrcf	UART2_RXReg,f		;belptets

u2rxnext	incf	UART2_RXState,f		;kvetkez llapot

u2rxend

;
; PC fel men gyr kezelse
; ---------------------------
;
;
;
;	


		movf	TOPC_WPtr,w		;van mit adni?
		subwf	TOPC_RPtr,w
		btfsc	STATUS,Z
		bra	topcend			;nincs

		movf	UART2_TXState,w		;lehet adni?
		sublw	.40
		btfss	STATUS,Z
		bra	topcend			;nem

		movff	FSR0L, IRQ_FSRTemp	; FSR mentse

		movff	TOPC_RPtr, FSR0L	; tovbbtand byte cme

		movff	INDF0, UART2_TXReg	; kldend byte az adregiszterbe

		clrf	UART2_TXState		;s ads mehet

		incf	TOPC_RPtr,w		;buffer olvass mutat nvekszik
		andlw	.31
		addlw	TOPC_Buffer
		movwf	TOPC_RPtr

		movff	IRQ_FSRTemp, FSR0L	;FSR visszatlts

topcend		bsf	LED
		btfsc	LED_Cntr, .7
		bcf	LED

		incfsz	LED_Cntr,w
		movwf	LED_Cntr

irqend		movf	IRQ_WTemp, w			;W visszallts
		movff	IRQ_STATUSTemp, STATUS		;STATUS visszallts

		retfie


;
; PC fel men gyr alaphelyzetbe lltsa
; -----------------------------------------
;
;


TOPC_Reset

		movlw	TOPC_Buffer		;a buffer elejn ll
		movwf	TOPC_WPtr		;az rs
		movwf	TOPC_RPtr		;s az olvass mutat is

		return

;
; 1 byte a PC fel men gyrbe
; -----------------------------
;
; W-ben a kldend byte
;


TOPC_PutCh

		movwf	TOPC_DataTemp		; kldend adat mentse

		movff	FSR0L, TOPC_FSRTemp	; FSR mentse

		movff	TOPC_WPtr, FSR0L	; gyrbe berand byte cme


	; olvasst nem rheti utol!!


		movff	TOPC_DataTemp, INDF0		; adat a gyrbe

		incf	TOPC_WPtr, w			;buffer rs mutat nvekszik
		andlw	0x1F
		addlw	TOPC_Buffer
		movwf	TOPC_WPtr

		movff	TOPC_FSRTemp, FSR0L		;FSR visszatlts

		return

;
; Fprogram - Inicializlsok
; ---------------------------
;

start		clrf	RCON			; prioritsi szintek tiltsa
		clrf	INTCON2
		clrf	INTCON3
		clrf	PIE1
		clrf	PIE2
		clrf	IPR1
		clrf	IPR2

		clrf	LATA
		setf	TRISA

		movlw	B'01000100'
		movwf	LATB
		movlw	B'00111010'
		movwf	TRISB

		movlw	B'11001000'
		movwf	T0CON

		clrf	ADCON0
		setf	ADCON1

		comf	PORTA, w
		andlw	0x0F
		movwf	XN_Addr

;		movlw	.2			;9600 baud
;		movwf	UART2_Baud
;		movwf	UART2_BaudCntr

		rcall	TOPC_Reset		;PC fel men gyr alapllapot

		rcall	FROMPC_Reset		;PC vtel alapllapot

		rcall	XN_ResetRX		;XpressNet vtelen indulunk

		rcall	DELAY

	;itt majd vrni kell egy kis vteli csendre!!!

		clrf	LED_Cntr

		movlw	.40
		movwf	UART2_TXState		;soft UART TX alapllapot
		clrf	UART2_RXState		;soft UART RX alapllapot
	
		bsf	INTCON,TMR0IE		;TMR0L megszakts engedlyezs
		bsf	INTCON,TMR0IF		;megszakts azonnal, amint GIE=1
		bsf	INTCON,GIE		;megszakts most mr mehet

		movlw	B'01100100'
;			  ||||||||-------------- TX9D, 9th transmit bit
;			  |||||||--------------- TRMT, 1 - TSR empty
;			  ||||||---------------- BRGH, 1 hihg speed mode
;			  |||||----------------- unimplemented, read as 0
;			  ||||------------------ SYNC, 0 - Asyncronous mode
;			  |||------------------- TXEN, 1 - Transmit enable
;			  ||-------------------- TX9, 1 - select 9 bit transmission
;			  |--------------------- CSRC, do not care in async mode

		movwf	TXSTA

		movlw	.39			;(OSCFQ/(.16*.62500))-1	;62500 baud
		movwf	SPBRG

		movlw	B'11010000'
;			  ||||||||-------------- RX9D, 9th received bit
;			  |||||||--------------- OERR, 1 - Overrun error
;			  ||||||---------------- FERR, 1 - Framing error
;			  |||||----------------- ADDEN, 1 - enable address detect
;			  ||||------------------ CREN, 1 - RX enable
;			  |||------------------- SREN, do not care in async mode
;			  ||-------------------- RX9, 1 - select 9 bit reception
;			  |--------------------- SPEN, 1 - enable serial port

		movwf	RCSTA

		clrf	BAUDCTL

		clrf	T1CON
		clrf	T2CON
		clrf	CCP1CON



;
; PC fell jv adatok kezelse
; -----------------------------
;
;

LOOP		movf	FROMPC_BytesLeft, f		;ha a PC-tl jv csomagba mr nem
;		btfsc	STATUS, Z			;kell tbb byte, akkor az adott csomag
;		bra	pcend				;XpressNet tovbbtsra vr...
		bz	pcend

	;lehetne timeout, hogy meddig vrhat....
	;br ezt inkbb az xn oldalnak kne nzni, hogy jn-e slot...

		movlw	.40				;a PC fell jtt adat?
		subwf	UART2_RXState,w
;		btfsc	STATUS,Z
;		bra	pcstore				;akkor troljuk
		bz	pcstore

		incfsz	FROMPC_BytesLeft,w		;csomag folyamatban a PC fell?
		bra	pcpinp
		bra	pcend				;ha nincs, akkor vrunk...

pcpinp		;movf	FROMPC_ToutCntr, w		;timeout?
;		btfss	STATUS,Z
		tstfsz	FROMPC_ToutCntr
		bra	pcend				;nem

		movlw	1				;timeout hiba klds a PC-nek
		rcall	TOPC_PutCh
		movlw	1
		rcall	TOPC_PutCh
		movlw	0
		rcall	TOPC_PutCh

		rcall	FROMPC_Reset			;s vtelt jraindtjuk

		bra	pcend


pcstore		movf	FROMPC_BufferPtr, w		;bufferbe
		movwf	FSR0L
		movf	UART2_RXReg, w			;a bejv byte-ot
		movwf	INDF0				;troljuk
		xorwf	FROMPC_XorByte, f		;s xor byte-ot is szmoljuk


		incf	FROMPC_BufferPtr, f		;buffer mutat nvekszik

		movlw	0xFF				;timeout szmll trlse
		movwf	FROMPC_ToutCntr

		clrf	UART2_RXState			;s vtel mehet tovbb

		incfsz	FROMPC_BytesLeft, w		;header byte?
		bra	nhfpc

		movf	INDF0,w				;akkor abban van
		andlw	.15
		addlw	.2
		movwf	FROMPC_BytesLeft		;a csomag hossza!
		movwf	FROMPC_ToXN			;XpressNet bufferbe msolshoz is kell mg

nhfpc		decfsz	FROMPC_BytesLeft, f		;kell mg adat a csomaghoz?
		bra	pcend				;igen, vrjuk a tbbit...

		bsf	UART2_CTS			;CTS tilts, mert teljes csomag megrkezett


	;xor byte ellenrzs
	;(mg) nem tudom, mi van akkor, ha nem j....
	;elmletileg nem is fontos...



		movf	FROMPC_Buffer,w			;header=11110000? (version info req)
		sublw	B'11110000'
;		btfss	STATUS,Z			;ha nem,
;		bra	pcend				;akkor majd megy tovbb XpressNet fel
		bnz	pcend

		movlw	002h				;verzi inf visszaklds
		rcall	TOPC_PutCh
		movlw	030h
		rcall	TOPC_PutCh
		movlw	001h
		rcall	TOPC_PutCh
		movlw	033h
		rcall	TOPC_PutCh

		rcall	FROMPC_Reset			;csomagot felfdolgoztuk

pcend




;
; Xpressnet
; ---------
;
;


		btfss	XN_State, 7			;XpressNet ads/vtel?
		bra	xntx



;
; Xpressnet fell jv adatok kezelse
; ------------------------------------
;
;



		btfss	PIR2, TMR3IF			;automatikus vlasz idzt letelt?
		bra	naansw

		movlw	1				;akkor automatikus vlasz a PC-nek
		rcall	TOPC_PutCh
		movlw	4
		rcall	TOPC_PutCh
		movlw	5
		rcall	TOPC_PutCh

		bcf	T3CON,TMR3ON			;vlasz idzt tilts
		bcf	PIR2,TMR3IF


naansw		btfss	RCSTA, OERR			;overrun hiba?
		bra	xnnovr				;(aminek csak a lassssgunk lehet az oka...)

		bcf	RCSTA, CREN			;vtel tilts a hiba trlshez

		rcall	XN_ResetRX			;majd jra engedlyezs

		bra	xnend				;s vrunk jabb adatra...


xnnovr		btfss	PIR1, RCIF			;jtt adat?
		bra	xnend				;ha nem, akkor vrunk...


		btfsc	RCSTA, RX9D			;call byte jtt?
		bra	xncb

		incfsz	XN_BytesLeft, w			;call byte-ra vrunk?
		bra	xnpbyte

		movf	RCREG, w			;ha igen, akkor RCREG rts
		bra	xnend				;s vrunk tovbb...


xncb		movf	RCREG, w			;rcall byte
		movwf	XN_rcallByte			;trols


	;parits ellenrzs


		andlw	.31				;broadcast?
;		btfsc	STATUS, Z
;		bra	xncbok
		bz	xncbok

		subwf	XN_Addr,w			;vagy a mi cmnk?
;		btfss	STATUS,Z
;		bra	xnend				;ha nem akkor vrunk...
		bnz	xnend


		movf	XN_rcallByte, w			;ack request?
		andlw	B'01100000'
;		btfsc	STATUS, Z
;		bra	xnackrq
		bz	xnackrq

		sublw	B'01000000'			;normal inquiry?
;		btfsc	STATUS,Z
;		bra	xninq
		bz	xninq

xncbok		movlw	0xFE				;rcall byte megvan
		movwf	XN_BytesLeft
		bra	xnend				;vrjuk a csomag tbbi byte-jt...


xnackrq		movlw	0x20				;ack
		movwf	XN_Buffer
		movlw	0x20
		movwf	XN_Buffer+1

		movlw	1				;PC-nek is megy a hibazenet
		rcall	TOPC_PutCh
		movlw	3
		rcall	TOPC_PutCh
		movlw	2
		rcall	TOPC_PutCh

		rcall	XN_ResetTX			;ads indulhat

		bra	xntx


xninq	
		clrf	LED_Cntr

		movf	FROMPC_BytesLeft, f		;PC-fell van adat?
;		btfsc	STATUS,Z			;ha nincs,
;		bra	xnpc
		bz	xnpc		

		rcall	XN_ResetRX			;akkor vtel jra alapra
		bra	xnend				;s vrunk tovbb...


xnpc		movlw	FROMPC_Buffer			;PC-tl jtt adatok kezdete
		movwf	FSR0L

pctoxn		movf	INDF0, w			;PC bufferbl egy byte
		bsf	FSR0L, 5
		movwf	INDF0				;az XpressNet bufferbe
		bcf	FSR0L, 5

		incf	FSR0L, f			;kvetkez byte

		decfsz	FROMPC_ToXN, f
		bra	pctoxn


		rcall	FROMPC_Reset			;PC vtel jra engedlyezs

		rcall	XN_ResetTX			;XpressNet ads pedig indulhat

		bra	xntx


xnpbyte		movf	XN_BufferPtr, w			;bufferbe
		movwf	FSR0L
		movf	RCREG, w			;a bejv byte-ot
		movwf	INDF0				;troljuk

		incf	XN_BufferPtr, f			;buffer mutat nvekszik

		movlw	0xFE				;header byte?
		subwf	XN_BytesLeft, w
;		btfss	STATUS, Z
;		bra	nhxn
		bnz	nhxn

		movf	INDF0, w			;akkor abban van
		andlw	.15
		addlw	.2
		movwf	XN_BytesLeft			;a csomag hossza!

nhxn		decfsz	XN_BytesLeft, f			;kell mg adat a csomaghoz?
		bra	xnend				;igen, vrjuk a tbbit...


	;xor byte ellenrzs


		movf	XN_Buffer, w			;byteok szma a csomagban
		andlw	.15
		addlw	.2
		movwf	XN_BytesLeft

		movlw	XN_Buffer			;csomag kezdete
		movwf	FSR0L

xntopc		movf	INDF0, w			;egy byte a csomagbl
		rcall	TOPC_PutCh			;megy a PC-nek

		incf	FSR0L, f			;kvetkez byte cme

		decfsz	XN_BytesLeft, f			;egy byte tkldve
		bra	xntopc


		bcf	T3CON, TMR3ON			;vlasz idzt tilts
		bcf	PIR2, TMR3IF

		rcall	XN_ResetRX			;vtel jra alapra
		bra	xnend				;s vrunk tovbb...



;
; Xpressnet ads
; --------------
;
;


xntx		btfsc	TXSTA,TRMT			;res az UART ad regisztere?
		bra	xnlehtx
		bra	xnend				;ha nem, akkor vrunk

xnlehtx		;movf	XN_BytesLeft,	f		;van mg adni val byte?
		;btfss	STATUS,Z
		;bra	xnleftt
		tstfsz	XN_BytesLeft
		bra	xnleftt

		clrf	TMR3L
		clrf	TMR3H				;automatikus vlasz idzt engedlyezs
		movlw	B'10010001'
		movwf	T3CON

		rcall	XN_ResetRX			;ha utols is elment, akkor visszavltunk vtelre
		bra	xnend

xnleftt 	movf	XN_BufferPtr, w			;bufferbl
		movwf	FSR0L
		movf	INDF0, w			;egy byte
		movwf	TXREG				;adsa

		incf	XN_BufferPtr, f			;buffer mutat nvekszik

		decf 	XN_BytesLeft, f			;egy byte leadva

xnend		bra	LOOP




;
; PC vtel alaphelyzetbe llts
; ------------------------------
;
;
;



FROMPC_Reset

		movlw	0xFF
		movwf	FROMPC_BytesLeft		;PC-tl jv csomag hossza mg ismeretlen
		movlw	FROMPC_Buffer			;vteli buffer az elejn ll
		movwf	FROMPC_BufferPtr
		clrf	FROMPC_XorByte			;xor byte alaprtk

		bcf	UART2_CTS			;CTS engedlyezs

		return


;
; XpressNet ads alaphelyzetbe llts
; ------------------------------------
;
;
;



XN_ResetTX

		clrf	XN_State			;ads md

		bsf	XN_DE				;RS485 ads engedlyezs

		bcf	RCSTA,CREN			;uart vtel tilts (visszhang miatt!)

		movf	XN_Buffer, w			;kldend byteok szma
		andlw	.15
		addlw	.2

		bra	xnrcom				;a tbbi kzs...


;
; XpressNet vtel alaphelyzetbe llts
; -------------------------------------
;
;
;


XN_ResetRX

		movlw	.128				;vtel md
		movwf	XN_State

		bcf	XN_DE				;RS485 ads tilts

		bsf	RCSTA, CREN			;uart vtel engedlyezs

		clrf	XN_XorByte			;xor byte alaprtk

		movlw	0xFF				;ismeretlen a venni kvnt
xnrcom		movwf	XN_BytesLeft			;byteok szma

		movlw	XN_Buffer			;buffer mutat az elejre
		movwf	XN_BufferPtr

		return



;------------------------------------------------------------
; trash
;


DELAY		clrf	Cntr1
		clrf	Cntr2

dly		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop

		decfsz	Cntr1, f
		bra	dly
		decfsz	Cntr2, f
		bra	dly

		return






	END
	

